
import App from "light"
import Http from '@/common/httpclient'
import config from "@/config"
let Stratages = null;
let TrackWay = null;
let installed = false;
var toString = Object.prototype.toString
function typeOf (val) {
  return toString.call(val).slice(8, -1).toLowerCase()
}
const processEvent = function (options) {//数据进行处理，并最后向埋点服务器发出埋点接口调用。
  let { location, context, paramsObj, contextType, props } = options
  let Cases = {
    pagename: App.router.app.$route.path,
    sourcepage: App.sourcepage,
    duration: Math.round(((+new Date) - App.mountedtime) / 1000)
  }
  let eventId = typeof location.eventId === 'function' ? location.eventId(context, paramsObj, App.router.app.$route.path) : location.eventId;
  let matchSwitch = true;
  if (location.switch) {//需要进行条件switch判断。
    matchSwitch = location.switch.some((condition) => {
      let match = Object.keys(condition.case).every((cskey) => {
        if (typeof condition.case[cskey] == 'string') {
          return Cases[cskey] === condition.case[cskey];
        } else {
          return condition.case[cskey].value == paramsObj[cskey] 
        }
      })
      if (match && condition.then) {
        let typeofThen = typeOf(condition.then);
        if (typeofThen === 'object') {
          Object.keys(condition.then).forEach((key) => {
            props[key] = condition.then[key]
          })
        } else if (typeofThen === 'array') {
          condition.then.forEach((prop) => {
            if (typeof prop === 'string') {
              props[prop] = Cases[prop]
            } else if (prop.value) {
              props[prop.key] = prop.value
            }
          })
        } 
      }
      return match;
    })
  }
  if (!matchSwitch) {
    return;
  }
  if (location.props) {//
    location.props.forEach(prop => {
      if (typeof prop === 'string') {
        props[prop] = Cases[prop]
      } else if (prop.value) {
        props[prop.key] = prop.value
      } else if (prop.param) {
        props[prop.key] = paramsObj[prop.param]
      } else if (prop.mapper) {
        try {
          if (contextType !== 'response' || context.successful) {
            props[prop.key] = prop.mapper(context, paramsObj)
          }
        } catch(e){}
      } else if (prop.contextkey) {
        if (contextType !== 'response' || context.successful) {
          // 含有.则逐层获取
          let keys = prop.contextkey.split(".")
          let index = 0;
          let ctx = context;
          try {
            while(keys[index]) {
              ctx = ctx[keys[index]];
              index++;
            }
            props[prop.key] = typeof ctx === 'object' ? ctx.toString() : ctx
          } catch(e) {}
        }
      }
    })
  }
  Tracker.track(eventId, props, options)//事件ID，埋点数据，
}
let lastTrackInfo;
let lastPageScreen;
const Tracker = {
  track (eventId, props, options) {
    // 刨除重复调用
    if (lastTrackInfo && lastTrackInfo.eventId === eventId && options && options.contextType === 'response') {
      if (options.context.successful == lastTrackInfo.options.context.successful && options.context.message === lastTrackInfo.options.context.message) {
          return;
      }
    }
    Stratages.beforeTrack && Stratages.beforeTrack(lastTrackInfo, eventId, props, options);
    TrackWay.track(eventId, props);
    Stratages.afterTrack && Stratages.afterTrack(lastTrackInfo, eventId, props, options)
    lastTrackInfo = { eventId, props, options }
  },
  trackHttp ({url, params, response, _autotrack = false}, properties) {//拦截接口请求
    let location;
    if (_autotrack) {
      location = Stratages.urlMapper[url];
    }
    if (location) {
      response = response || {} 
      let props = {//发送服务时的入参基本对象。
        issuccess: response.successful || false,
        errormessage: response.successful ? '' : (response.message || '')
      }
      let eventId;
      if (typeof location == 'string') {
        eventId = location;
        this.track(eventId, props)
      } else {
        let paramsObj = {}
        params.split("&").reduce((obj, str) => {
          let pair = str.split("=");
          obj[pair[0]] = decodeURIComponent(pair[1] || '')
          return obj;
        }, paramsObj)
        let locationArr;
        if (typeOf(location) === 'object') {
          locationArr = [location]
        } else {
          locationArr = location;
        }
        locationArr.forEach((newlocation) => {
          if (newlocation.context) {
            newlocation.context().then((result) => {
              let context = result.default._private.formData ? result.default._private.formData : result.default._private;
              processEvent({
                location: newlocation, context, paramsObj, props
              });
            })
          } else {
            processEvent({
              location: newlocation, context: response, paramsObj, contextType: 'response', props
            });
          }
        })
      }
    }
  },
  trackPageView (from, to, properties) {
    const defaultProps = {
      registsuccesstopage: to.path,
      sourcepage: from.path
    }
    let view = to.path.replace(/^\//, "");
    let location = Stratages.viewMapper[view];
    if (!location) {
      Object.keys(Stratages.viewMapper).forEach(key => {
        if (key[0] === '^' && new RegExp(key).test(view)) { // ^ 开头视为正则匹配
          location = Stratages.viewMapper[key];
        }
      })
    }
    // 正则匹配
    if (!location) {
      view = from.path.replace(/^\//, "");
      location = Stratages.viewMapper[view];
      if (!(location && location.mode === 'leave')) {
        return;
      }
    } else {
      if (typeof location == 'object' && location.mode === 'leave') {
        return;
      }
    }
    if (location) {
      let eventId;
      let props = {}
      if (typeof location === 'string') {
        eventId = location;
        props.sourcepage = from.path;
      } else {
        eventId = location.eventId;
        if (location.props) {
          location.props.forEach(prop => {
            if (typeof prop === 'string') {
              props[prop] = defaultProps[prop]
            } else if (prop.value) {
              props[prop.key] = prop.value
            }
          })
        }
      }
      this.track(eventId, props)
    }
  },
  trackClick (trackId, context) {
    let location = Stratages.clickMapper[trackId];
    if (location) {
      let props = {
        issuccess: true,
        errormessage: ''
      }
      let eventId;
      if (typeof location == 'string') {
        eventId = location;
        this.track(eventId, props)
      } else {
        processEvent({
          location, 
          context: context.formData ? context.formData : context, 
          paramsObj: {
            applysum: context.currentValue,
            currentCard: context.currentCard,
            // 下面是定投使用
            protocolperiodunit: context.cycleUnit,
            tradeperiod: context.cycle && context.cycle.index === 1 ? '2' : '1',
            fixdate: context.cycleUnit == 2 ? context.firstTradeDate.substr(-2) : context.fixDate
          },
          props
        });
      }
    }
  },
  /**
   * 页面可见后发送埋点信息
   */
  trackPageScreen (url, properties) {
    if (Stratages.trackPageScreen) {
      // 去重
      if (lastPageScreen !== url) {
        lastPageScreen = url;
        TrackWay.trackViewScreen && TrackWay.trackViewScreen(url, {
          $title: properties.title
        });
      }
    }
  },
  trackCustEvent (type, properties) {
    let location = Stratages.customTrackMapper[type];
    if (location) {
      let props = {};
      if (location.props) {
        location.props.forEach(prop => {
          if (typeof prop === 'string') {
            props[prop] = properties[prop]
          } else if (prop.value) {
            props[prop.key] = prop.value
          }
        })
      }
      Tracker.track(location.eventId, props);
    }
  }
}
export default {
  install(Vue, options) {//for example: newealth-project-extends.js/service/plugin
    if (installed) return;
    Stratages = options.stratages
    TrackWay = options.tracker
    installed = true;
    // 配置了策略和埋点消费者
    if (TrackWay && Stratages) {
      TrackWay.init(options)
      Stratages.tracker = TrackWay
      /**
       * 捕捉：
       * 1. 按钮点击，只使用Button.vue组件。
       * 2. 页面可见resume。页面跳转完成，记录相关信息。
       * 3. 页面路由，跳转前。
       * 4. 接口拦截。
       */
      App.on("buttonClick", function (event) {
        let { trackId, context } = event;
        Tracker.trackClick(trackId, context);
      })
      App.on("pageViewShow", function (event) {
        let matchIndexNotFirst = false;
        if (!event.title) {
          config.indexPage.some((page, index) => {
            if (new RegExp(`\\/${page.url}($|\\?)`).test(event.path)) {
              event.title = page.text;
              if (index !== 0) {
                matchIndexNotFirst = true;
              }
              return true;
            }
            return false;
          })
        }
        // app中首页不走此处，而是交由switchTab
        if (weex.config.env.platform !== 'Web' && matchIndexNotFirst) return;
        Tracker.trackPageScreen(event.path, {
          title: event.title
        })
      })
      // 路由跳转拦截器
      // web拦截route
      if (weex.config.env.platform === 'Web') {
        App.filter("view", function (from, to, next) {//相当于vue-router.beforeEach
          try {
            Tracker.trackPageView(from, to);
            App.sourcepage = from.path;//为了标识下一次请求的来源页面（起始页面）
          } catch(e) {
            if (process.env.NODE_ENV === 'dev') {
              console.error(e)
            }
          }
          next();
        })
      } else {//App场景执行。捕捉页面切换(切换前发生)。
        // app拦截navigate
        App.filter("navigate", function (view, params, options, next) {//相当于准备跳转页面。
          params.sourcepage = encodeURIComponent(App.router.app.$route.path)
          try {
            Tracker.trackPageView(App.router.app.$route, view);
          } catch(e) {
            if (process.env.NODE_ENV === 'dev') {
              console.error(e)
            }
          }
          next();
        })
        App.on("switchTab", function (event) {//相当于首页切换
          let { currentIndex, lastIndex } = event;
          try {
            let indexPage = config.indexPage
            Tracker.trackPageScreen('/' + indexPage[currentIndex].url, {
              title: indexPage[currentIndex].text
            })
          } catch(e) {
            if (process.env.NODE_ENV === 'dev') {
              console.error(e)
            }
          }
        })
        App.on("backToIndex", function (data) {
          let view;
          if (typeof data.viewIndex != 'undefined') {
            view = config.indexPage[data.viewIndex];
            Tracker.trackPageScreen('/' + view.url, {
              title: view.text
            })
            Tracker.trackPageView(App.router.app.$route, {
              path: '/' + view.url
            });
          }
        })
      }
      // 接口拦截
      Http.interceptors.response.push(function (options) {
        try {
          Tracker.trackHttp({
            ...options,
            _autotrack: true
          });
        } catch(e) {
          if (process.env.NODE_ENV === 'dev') {
            console.error(e)
          }
        }
      })
      App.on("trackCustEvent", function (event) {
        Tracker.trackCustEvent(event.type, event.payload)
      })
      Vue.prototype.$tracker = Tracker
    }
  }
}