let sensors = null;
export default {
  lastEventId: null,
  platform: 'app',
  isEnable: function () {
    let isWeb = weex.config.env.platform === 'Web';
    return (!isWeb && this.platform != 'web') || (isWeb && this.platform != 'app')
  },
  init (options) {
  
    this.platform = options.platform;
    // web端且开启h5埋点
    if (this.platform !== 'app' && weex.config.env.platform === 'Web') {//只适用web端，
      this.enableWebTrack(options.webOptions);
    }
    sensors = weex.requireModule('WeexSensorsDataAnalyticsModule');
  },
  enableWebTrack (options) {//官网固定代码
    (function(para) {
      var p = para.sdk_url, n = para.name, w = window, d = document, s = 'script',x = null,y = null;
      if(typeof(w['sensorsDataAnalytic201505']) !== 'undefined') {
          return false;
        }
      w['sensorsDataAnalytic201505'] = n;
      w[n] = w[n] || function(a) {return function() {(w[n]._q = w[n]._q || []).push([a, arguments]);}};
      var ifs = ['track','quick','register','registerPage','registerOnce','clearAllRegister','trackSignup', 'trackAbtest', 'setProfile','setOnceProfile','appendProfile', 'incrementProfile', 'deleteProfile', 'unsetProfile','identify','login','logout','trackLink'];
      for (var i = 0; i < ifs.length; i++) {
        w[n][ifs[i]] = w[n].call(null, ifs[i]);
      }
      if (!w[n]._t) {
        x = d.createElement(s), y = d.getElementsByTagName(s)[0];
        x.async = 1;
        x.src = p;
        x.setAttribute('charset','UTF-8');
        w[n].para = para;
        y.parentNode.insertBefore(x, y);
      }
    })(options); 
    // WeexSensorsDataAnalyticsModule定义
    weex.registerModule('WeexSensorsDataAnalyticsModule', {//App里增加了神策sdk，对外暴露了WeexSensorsDataAnalyticsModule。为了web端统一调用逻辑，这里手动生成兼容的api。
      track (eventId, properties) {
        window.sensors && window.sensors.track(eventId, properties)
      },
      login (userId) {
        window.sensors && window.sensors.login(userId);
      },
      profileSet (properties) {
        window.sensors && window.sensors.setProfile(properties)
      },
      profileSetOnce (properties) {
        window.sensors && window.sensors.setOnceProfile(properties)
      },
      trackViewScreen (url, properties) {
        window.sensors && window.sensors.quick('autoTrack', properties)
      }
    })
  },
  /**
   * 埋点
   * @param {string} eventId 事件Id
   * @param {object} properties 事件参数
   * @param {string} properties.sourcepage 页面来源
   * @param {string} properties.issuccess 是否成功
   * @param {string} properties.errormessage 失败信息
   */
  track(eventId, properties) {// App和web都统一调用这个方法。
    this.lastEventId = eventId;
    if (this.isEnable()) {
      sensors && sensors.track(eventId, properties)
    } 
  },
  // 登录
  login (userId) {
    if (this.isEnable()) {
      sensors && sensors.login(userId);//神策sdk方法。为了解决未登录和登录后，埋点数据的不同。
    }
  },
  // 退出登录
  logout () {
    sensors.logout(true);//当前代码中未调用。神策sdk方法。为了解决未登录和登录后，埋点数据的不同。
  },
  /**
   * 设置用户信息
   * @param {object} properties 
   */
  profileSet (properties) {
    if (this.isEnable()) {
      sensors && sensors.profileSet(properties)
    }
  },
  // 注册、开户时
  profileSetOnce (properties) {
    if (this.isEnable()) {
      sensors.profileSet(properties)//当前代码中没有调用，代码可能有误，需要改成profileSetOnce。
    }
  },
  /**
   * 切换页面的时候调用
   * @param {string} url 
   * @param {object} properties 
   */
  trackViewScreen (url, properties) {
    if (this.isEnable()) {
      sensors && sensors.trackViewScreen(url, properties);
    }
  }
}